@@ -31,10 +31,11 @@
 #include <iostream>
 
 LDAPConnection::LDAPConnection(const std::string &server) : m_ldap() {
+    int ret = 0;
 #ifndef SOLARIS_LDAP_KERBEROS
     std::string ldap_url = "ldap://" + server;
     VERBOSEldap("calling ldap_initialize");
-    int ret = ldap_initialize(&m_ldap, ldap_url.c_str());
+    ret = ldap_initialize(&m_ldap, ldap_url.c_str());
 #else
     VERBOSEldap("calling ldap_init");
     m_ldap = ldap_init(flags->server.c_str(), LDAP_PORT);
@@ -417,7 +418,7 @@ int ldap_set_supportedEncryptionTypes(const std::string &dn, msktutil_flags *fla
         VERBOSE("DEE dn=%s old=%d new=%d\n",
                 dn.c_str(), flags->ad_supportedEncryptionTypes, flags->supportedEncryptionTypes);
 
-        int ret = ldap_simple_set_attr(flags->ldap.get(), dn, "msDs-supportedEncryptionTypes",
+        ret = ldap_simple_set_attr(flags->ldap.get(), dn, "msDs-supportedEncryptionTypes",
                                        supportedEncryptionTypes);
 
         if (ret == LDAP_SUCCESS) {
@@ -738,6 +739,10 @@ void ldap_check_account(msktutil_flags *flags)
             mesg = ldap_first_entry(flags->ldap->m_ldap, mesg);
             std::vector<std::string> vals = flags->ldap->get_all_vals(mesg, "servicePrincipalName");
             for (size_t i = 0; i < vals.size(); ++i) {
+	        // translate HOST/ to host/
+		if (vals[i].compare(0, 5, "HOST/") == 0) {
+		  vals[i].replace(0, 5, "host/");
+                }
                 flags->ad_principals.push_back(vals[i]);
                 VERBOSE("  Found Principal: %s", vals[i].c_str());
             }
