@@ -104,7 +104,7 @@ int finalize_exec(msktutil_exec *exec)
 {
     msktutil_flags *flags = exec->flags;
     int ret;
-
+    
     char *temp_realm;
     if (flags->realm_name.empty()) {
         if (krb5_get_default_realm(g_context.get(), &temp_realm)) {
@@ -302,6 +302,10 @@ void do_help() {
     fprintf(stdout, "                         This option is mutually exclusive with --user-creds-only.\n");
     fprintf(stdout, "  -h, --hostname <name>  Use <name> as current hostname.\n");
     fprintf(stdout, "  -k, --keytab <file>    Use <file> for the keytab (both read and write).\n");
+    fprintf(stdout, "  --keytab-auth-as <name>\n");
+    fprintf(stdout, "                         First try to authenticate to AD as principal <name>, using\n");
+    fprintf(stdout, "                         creds from the keytab, instead of using the account name\n");
+    fprintf(stdout, "                         principal or the host principal, etc.\n");
     fprintf(stdout, "  --server <address>     Use a specific domain controller instead of looking\n");
     fprintf(stdout, "                         up in DNS based upon realm.\n");
     fprintf(stdout, "  --server-behind-nat    Ignore server IP validation error caused by NAT.\n");
@@ -327,9 +331,10 @@ void do_help() {
     fprintf(stdout, "  --do-expire-password   Undisables (puts back to default) password expiration.\n");
     fprintf(stdout, "  --enctypes <int>       Sets msDs-supportedEncryptionTypes\n");
     fprintf(stdout, "                         (OR of: 0x1=des-cbc-crc 0x2=des-cbc-md5\n");
-    fprintf(stdout, "                                 0x4=rc4-hmac-md5 0x8=aes128-ctc-hmac-sha1\n");
+    fprintf(stdout, "                                 0x4=rc4-hmac-md5 0x8=aes128-cts-hmac-sha1\n");
     fprintf(stdout, "                                 0x10=aes256-cts-hmac-sha1)\n");
     fprintf(stdout, "                         Sets des-only in userAccountControl if set to 0x3.\n");
+    fprintf(stdout, "  --allow-weak-crypto    Enables the usage of DES keys for authentication\n");
     fprintf(stdout, "  --no-pac               Sets the service principal to not include a PAC.\n");
     fprintf(stdout, "  --disable-no-pac       Sets the service principal to include a PAC.\n");
     fprintf(stdout, "  -s, --service <name>   Adds the service <name> for the current host or the\n");
@@ -376,7 +381,7 @@ int execute(msktutil_exec *exec)
     } else if (exec->mode == MODE_CREATE || exec->mode == MODE_UPDATE || exec->mode == MODE_AUTO_UPDATE) {
         if (exec->mode == MODE_AUTO_UPDATE) {
             // Don't bother doing anything if the auth was from the keytab (and not e.g. default password), and the
-            if (exec->flags->auth_type == AUTH_FROM_SAM_KEYTAB) {
+            if (exec->flags->auth_type == AUTH_FROM_SAM_KEYTAB || exec->flags->auth_type == AUTH_FROM_EXPLICIT_KEYTAB) {
                 std::string pwdLastSet = ldap_get_pwdLastSet(exec->flags);
                 // Windows timestamp is in 100-nanoseconds-since-1601. (or, tenths of microseconds)
                 long long windows_timestamp = strtoll(pwdLastSet.c_str(), NULL, 10);
@@ -426,7 +431,7 @@ int execute(msktutil_exec *exec)
         return ret;
     } else if (exec->mode == MODE_PRECREATE) {
         // Change account password to default value:
-        flags->password = flags->samAccountName_nodollar;
+        flags->password = create_default_machine_password(flags->samAccountName);
         // Check if computer account exists, update if so, create if not.
         ldap_check_account(flags);
 
@@ -572,6 +577,12 @@ int main(int argc, char *argv [])
             continue;
         }
 
+        /* Re-activate DES encryption in fake krb5.conf */
+        if (!strcmp(argv[i], "--allow-weak-crypto")) {
+            exec->flags->allow_weak_crypto = true;
+            continue;
+        }
+
         /* Disable the PAC ? */
         if (!strcmp(argv[i], "--no-pac")) {
             exec->flags->no_pac = VALUE_ON;
@@ -709,6 +720,16 @@ int main(int argc, char *argv [])
             continue;
         }
 
+        if (!strcmp(argv[i], "--keytab-auth-as")) {
+            if (++i < argc) {
+                exec->flags->keytab_auth_princ = argv[i];
+            } else {
+                fprintf(stderr, "Error: No principal given after '%s'\n", argv[i - 1]);
+                goto error;
+            }
+	    continue;
+        }
+
         /* Display Verbose Messages */
         if (!strcmp(argv[i], "--verbose")) {
             do_verbose();
@@ -770,15 +791,29 @@ error:
 
 
 msktutil_flags::msktutil_flags() :
-    password(), ldap(), set_description(false), set_userPrincipalName(false),
-    dont_expire_password(VALUE_IGNORE), no_pac(VALUE_IGNORE), delegate(VALUE_IGNORE),
-    ad_userAccountControl(0), ad_enctypes(VALUE_IGNORE), ad_supportedEncryptionTypes(0),
+    password(),
+    ldap(),
+    set_description(false),
+    set_userPrincipalName(false),
+    no_reverse_lookups(false),
+    server_behind_nat(false),
+    set_samba_secret(false),
+    dont_expire_password(VALUE_IGNORE),
+    no_pac(VALUE_IGNORE),
+    delegate(VALUE_IGNORE),
+    ad_userAccountControl(0),
+    ad_enctypes(VALUE_IGNORE),
+    ad_supportedEncryptionTypes(0),
     enctypes(VALUE_IGNORE),
     /* default values we *want* to support */
     supportedEncryptionTypes(MS_KERB_ENCTYPE_RC4_HMAC_MD5 |
                              MS_KERB_ENCTYPE_AES128_CTC_HMAC_SHA1_96 |
                              MS_KERB_ENCTYPE_AES256_CTS_HMAC_SHA1_96),
-    auth_type(0), user_creds_only(false), password_expired(false)
+    auth_type(0),
+    user_creds_only(false),
+    use_service_account(false),
+    allow_weak_crypto(false),
+    password_expired(false)
 {}
 
 msktutil_flags::~msktutil_flags() {
